/**
* @파일명 : common_evCharge.js
* @작성자 : 네이버시스템-강보경
* @생성일 : 2021. 6. 22
* @메뉴명 : 공통 전기차 충전 관련 파일
*/

// API URL
var SSC_API = {
    joinUserInfo : {
        nm : 'SSCharge 회원 가입 등록'  ,
        url : 'vik/member/join/request',
    },
    uptUserInfo : {
        nm : 'SSCharge 회원 정보 변경'  ,
        url : 'vik/member/info/update',
    },
    userCardInfo : {
        nm : 'SSCharge 회원 카드 정보 조회'  ,
        url : 'vik/member/info/inquiry',
    },
    otpQr : {
        nm : 'OTP QR생성'  ,
        url : 'vik/auth/qr/otp/request',
    },
    qrRead : {
        nm : 'QR Read 처리 요청'  ,
        url :  'vik/auth/qr/read/request',
    },
    chargeSumList : {
        nm : '충전 완료 정보 합계 조회'  ,
        url :  'vik/charge/completion/sum/inquiry',
    },
    chargeInfoList : {
        nm : '충전 완료 정보 목록 조회'  ,
        url :  'vik/charge/v2/completion/list/inquiry',
    },
    kiaPointCriteriaInfo : {
        nm : '기아 멤버스 포인트 사용 기준 정보 조회'  ,
        url :  'vik/kia/members/criteria/inquiry',
    },
    uptKiaPointCriteria : {
        nm : '기아 멤버스 포인트 사용 기준 정보 변경'  ,
        url :  'vik/kia/members/criteria/update',
    },
    reqEvCardInfo : {
        nm : 'EV-교통카드 발급/변경/삭제 요청'  ,
        url :  'vik/card/issue/request',
    },
    /** 충전소 관련 API */
    reqEvChrgrResv : {
        nm : '충전기 예약 신청 요청'  ,
        url :  'vik/charger/reservation/request',
    },
    evChrgrResvReqList : {
        nm : '충전기 예약 신청 내역 조회'  ,
        url :  'vik/charger/reservation/inquiry',
    },
    canclEvChrgrResv : {
        nm : '충전기 예약 신청 취소'  ,
        url :  'vik/charger/reservation/cancel',
    },
    evChrgrResvSttusList : {
        nm : '충전기 예약 현황 조회'  ,
        url :  'vik/charger/reservation/status/inquiry',
    },
    possibletime : {
        nm : '충전기 예약 가능 시간 조회'  ,
        url :  'vik/charger/reservation/possibletime/inquiry',
    },
    reqResvStatus : {
        nm : '충전기 예약 신청 상태 조회'  ,
        url :  'vik/charger/reservation/request/status',
    },
    reqChrgLocation : {
        nm : '위치기준 추천 충전소 정보 요청'  ,
        url :  'vik/charger/core/v2/locationbase/inquiry',
    },
    basicChrgInfo : {
        nm : '충전소 기본 정보 조회'  ,
        url :  'vik/charger/core/v2/basic/inquiry',
    },
    progressChrgInfo : {
        nm : '충전 진행 중 확인요청'  ,
        url :  'vik/charge/progress/info/inquiry',
    }
}
// 충전타입목록
const EV_CHARGER_TYPE = ["DC차데모", "완속", "DC콤보", "AC상"];

// 카드타입목록
const EV_CARD_TYPE = ["EV", "EVT", "EV9"];


/**
 * @name        : isAvailableEvMembers
 * @description : EV 멤버스 관련 서비스를 이용할 수 있는지 검증
 * @date        : 2023.08.28
 * @author      : 디지털채널시스템팀-정성윤
 */
async function isAvailableEvMembers() {
    if (!isLoggedIn()) return false;
    if (!isEvCar()) return false;
    if (!(await isEvMembersMember())) return false;
    return true;
}

/**
 * @name        : isAvailableSSCharger
 * @description : SK 일렉링크 관련 서비스를 이용할 수 있는지 검증
 * @date        : 2023.08.28
 * @author      : 디지털채널시스템팀-정성윤
 */
async function isAvailableSSCharger(joinMessage) {
    if (!(await isAvailableEvMembers())) return false;
    if (!(await isSsChargerMember(joinMessage))) return false;
    return true;
}

/**
* @name        : isLoggedIn
* @description : 로그인 여부
* @date        : 2023.09.04
* @author      : 디지털채널시스템팀-정성윤
*/
function isLoggedIn() {
    if (!PGV?.loginInfo) {
        COMMON_FUNCTION.TOOL.POPUP.toggle(PATH_MANAGER.KIA_RNW_080_0000.path, null, [null, function(){
            ccspLogin();
        }]);
        return false
    }
    return true;
}

/**
* @name        : isEvCar
* @description : EV 자동차 여부
* @date        : 2023.08.28
* @author      : 디지털채널시스템팀-정성윤
*/
function isEvCar() {
    const selectedCar = COMMON_FUNCTION.TOOL.DATA.getCurrentScopeSelectCar();
    if (isEmpty(selectedCar)) {
        MPopup.alert("등록된 차량이 없습니다.")
        return false
    }

    if (selectedCar.memEvYn === "N") {
        MPopup.alert("EV 차량 전용 서비스입니다.");
        return false
    }

    return true;
}

/**
* @name        : isEvMembersMember
* @description : EV 멤버스 카드 보유 회원인지 검증
* @date        : 2023.08.28
* @author      : 디지털채널시스템팀-정성윤
*/
async function isEvMembersMember() {
    if (PGV.loginInfo.evMemRegYn === "Y") {
        return true;
    }

    const membersCardList = await COMMON_FUNCTION.TOOL.CARD.loadCardsList(true)
    if (evCardChk(membersCardList)) {
        return true;
    }

    return false;
}

/**
* @name        : isSsChargerMember
* @description : SK 일렉링크 회원인지 확인
* @date        : 2023.08.28
* @author      : 디지털채널시스템팀-정성윤
*/
async function isSsChargerMember(message) {
    const ssChargerCard = await COMMON_FUNCTION.TOOL.CARD.getSSChargerCard()

    if (ssChargerCard.resultCode !== '200') {
        MPopup.alert("일시적인 시스템 오류입니다. 고객센터로 문의 부탁드립니다.\r\n※ SK일렉링크 고객센터(1566-1704, 24시간 운영)");
        return false;
    }

    if (ssChargerCard.evMemRegYn === "N") {
        MPopup.confirm({
            message: message || "충전 로밍 서비스를 사용하기 위해서는 SK일렉링크 회원가입이 필요합니다. 회원가입 화면으로 이동하시겠습니까?",
            buttons: ["확인", "취소"],
            callback: function (idx) {
                if (idx === 0) {
                    MPage.html({
                        url: PATH_MANAGER.KIA_RNW_5E0_0000.path,
                        animate: "MODAL_UP"
                    });
                }
            }
        });
        return false;
    }

    if (ssChargerCard.evMemTermsYn === "Y") {
        MPopup.confirm({
            message: "충전 로밍 서비스 약관이 변경되었습니다.\n서비스 이용을 위한 약관 동의 부탁드립니다.",
            buttons: ["확인", "취소"],
            callback: function (idx) {
                if (idx === 0) {
                    MPage.html({
                        url: PATH_MANAGER.KIA_RNW_5E0_0000.path,
                        param: {isChgAgree: "Y"},
                        animate: "MODAL_UP"
                    });
                }
            }
        });
        return false;
    }

    return true;
}

/**
* @name        : isGreenPassSubscriber
* @description : 그린패스 서비스를 구독중인지 확인
* @date        : 2023.08.28
* @author      : 디지털채널시스템팀-정성윤
*/
async function isGreenPassSubscriber(message) {
    const ssChargerCard = await COMMON_FUNCTION.TOOL.CARD.getSSChargerCard()
    if (ssChargerCard.resultCode !== '200') {
        MPopup.alert("일시적인 시스템 오류입니다. 고객센터로 문의 부탁드립니다.\r\n※ SK일렉링크 고객센터(1566-1704, 24시간 운영)");
        return false;
    }
    // productStatus O : 구독 중 , X : 미구독, OE : 해지 예약 , OC 변경 예약
    if (ssChargerCard.productStatus === "O") {
        return true;
    }

    return false;
}

/**
* @name        : evCardChk
* @description : ev카드 조회
* @date        : 2021.07.08
* @author      : 네이버시스템-강보경
*/
var evCardChk = function(cardList){
    if(isEmpty(cardList)){
        //조회가 되지 않음 - 멤버십 없는경우
        MPopup.confirm({
            message : 'EV멤버십 회원이 아닙니다.\nEV멤버십에 가입하시겠습니까?',
            buttons : ["취소", "확인"],
            callback : function(idx){
                if(idx == 1){ // 확인
                    var _url = CONSTANT.LINK_URL.KIA_MOB_810_1042.URL;
                    MPage.activity({
                        className : "WebviewActivity",
                        param : {
                            method : CONSTANT.NET_MTHD.POST,
                            requestURL : _url,
                            postData : {
                                mbrId : PGV.loginInfo.mbrId
                            }
                        }
                    });
                }
            }
        });
        return false;
    }else {
        for(var i=0; i<cardList.length; i++){
            if(EV_CARD_TYPE.includes(cardList[i].crdTypeCd)){
                return cardList[i];
            }
        }

        //조회가 되지 않음 - 카드 없는경우 (EV 차량보유여부는 isSupportEvCharge 에서 체크함)
        MPopup.confirm({
            message : 'EV 멤버스 카드 발급이 가능합니다.\nEV멤버십 카드로 교체발급하시겠습니까?',
            buttons : ["취소", "확인"],
            callback : function(idx){
                if(idx == 1){ // 확인
                    MNet.httpSendAsIs({     // 카드 교체발급 조회
                        path : checkVerifyUrl(inf.card.pssblExchngList.url),
                        sendData : {
                            csetIfTypeCd : "04",
                            csetIfValue : PGV.loginInfo.csmrMgmtNo
                        },
                        callback : function(receivedData){
                            exWNHideIndicator();

                            var code = receivedData.resultCode;
                            var msg = receivedData.resultMsg;
                            var cardList = receivedData.cardExchngListResDto || [];
                            var chngCardList = receivedData.cardPrmnCarListResDto || [];

                            if(code == "200" && cardList.length > 0 && chngCardList.length > 0){
                                MPage.html({
                                    url : PATH_MANAGER.KIA_RNW_0D3_2220.path,
                                    param : {
                                        cardList : cardList,
                                        chngCardList : chngCardList
                                    }
                                });
                            }else{
                                MPopup.alert("교체 신청이 가능한 카드가 없습니다.\n기아멤버스 카드를 보유하였으나, 교체발급 신청이 되지 않는 경우, 고객센터 080-200-2000으로 연락해 주세요.");
                            }
                        }
                    });
                }
            }
        });
        return false;
    }
}

/*
* @name        : joinMem
* @description : SSCharge 회원 가입 등록
* @date        : 2021.06.28
* @author      : 네이버시스템-주해원
*
* -Request Parameter
*   ci                  : CI (암호화)
*   billkey             : 빌키 (나이스페이, 암호화)
*   name                : 이름 (암호화)
*   contactNo           : 연락처 (암호화)
*   vehicleType         : 차종 (암호화)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*/
var joinMem = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.joinUserInfo.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
 * @name        : regEvMng
 * @description : EV멤버스 등록
 * @date        : 2021. 7. 1
 * @author      : 네이버시스템-주해원
 */
var regEvMng = function(successFun){        // 이벤트여부 플래그 추가
    var _sendData = {
        apiId: "RNW-EVC-3001",
        mbrId: PGV.loginInfo.mbrId,
        entYn: "Y",
        //mbrUnqNo: PGV.loginInfo.csmrMgmtNo,
        cofAgrYn1: "Y",      // 서비스이용약관
        cofAgrYn2: "Y",      // 충전 로밍 제휴 서비스 이용 약관
        cofAgrYn3: "Y",      // 개인정보 수집
        cofAgrYn4: "Y",       // 제3자 (기아 -> ST)
        cofAgrYn5: "Y",       // 제3자 (ST -> 기아)
        evtYn: MData.param("evtYn") || "N"
    };

    MNet.httpSendAsIs({
        path : _API.SVR.EV.evmng.evMngReg,
        sendData : _sendData,
        callback : successFun
    });
};

/**
* @name        : updtMem
* @description : SSCharge 회원 정보 변경
* @date        : 2021.06.28
* @author      : 네이버시스템-주해원
*
* -Request Parameter
*   ci                  : CI (암호화)
*   billkey             : 빌키 (나이스페이, 암호화)
*   name                : 이름 (암호화)
*   contactNo           : 연락처 (암호화)
*   vehicleType         : 차종 (암호화)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*/
var updtMem = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.uptUserInfo.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
* @name        : evCardKey
* @description : [공통] STraffic 회원 카드 정보 조회 (EV-교통카드 번호)
* @date        : 2021.06.23
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   ci                  : CI (암호화)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*   evTrafficCardNo     : EV-교통카드번호
*   creditCardNo        : 결제카드번호 (마스킹)
*/
var evCardKey = function(param, successFun, failFun){

    MNet_SSC.httpSend({
        path : SSC_API.userCardInfo.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
* @name        : createOtpQr
* @description : OTP QR생성
* @date        : 2021.06.23
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   evTrafficCardNo     : EV-교통카드번호
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*   otpQrNo             : OTP QR 번호
*/
var createOtpQr = function(param, successFun, failFun){

    MNet_SSC.httpSend({
        path : SSC_API.otpQr.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
}

/**
* @name        : readQrRequest
* @description : QR Read 처리 요청
* @date        : 2021.06.23
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   evTrafficCardNo     : EV-교통카드번호
*   gid                 : 서비스그룹ID
*   bid                 : 기관ID
*   sid                 : 충전소ID
*   cid                 : 충전기ID
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*/
var readQrRequest = function(param, successFun, failFun){

    MNet_SSC.httpSend({
        path : SSC_API.qrRead.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
}

/**
* @name        : selectChargeSumList
* @description : 충전 완료 정보 합계 조회
* @date        : 2021.06.23
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   ci                  : CI (암호화)
*   startDate           : 조회 시작 일자 (yyyy-MM-dd)
*   endDate             : 조회 종료 일자 (yyyy-MM-dd)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*   evChargeAmount      : 충전량
*   totalCost           : 충전금액
*   averageUnitPrice    : 평균단가
*   pointUseCost        : 포인트 사용 금액
*   totalCnt            : 전체건수
*/
var selectChargeSumList = function(param, successFun, failFun){

    MNet_SSC.httpSend({
        path : SSC_API.chargeSumList.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
}

/**
* @name        : selectChargeInfoList
* @description : 충전 완료 정보 목록 조회
* @date        : 2021.06.23
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   ci                  : CI (암호화)
*   startDate           : 조회 시작 일자 (yyyy-MM-dd)
*   endDate             : 조회 종료 일자 (yyyy-MM-dd)
*   currentPageNo       : 현재 페이지 번호
*   recordsPerPage      : 출력할 데이터 갯수
*   sorting             : 정렬(A: 오름차순, D: 내림차순)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*   chargeUseInfo       : 충전 이용 정보
*   ㄴ   evChargeCompleteDate    : 충전완료일자( yyyy-MM-dd )
*   ㄴ   evStationName           : 충전소명
*   ㄴ   evChargeAmount          : 충전량 (kwh)
*   ㄴ   totalCost               : 충전금액
*   ㄴ   unitPrice               : 적용단가
*   ㄴ   billingTargetCost       : 청구대상금액
*   ㄴ   pointUseRequestCost     : 기아 멤버스 포인트 사용 요청금액
*   ㄴ   pointApprovalNo         : 기아 멤버스 포인트 승인번호
*   ㄴ   billingCompleteCost     : 청구 완료 금액
*   ㄴ   outstandingBalance      : 미청구 금액
*/
var selectChargeInfoList = function(param, successFun, failFun){

    MNet_SSC.httpSend({
        path : SSC_API.chargeInfoList.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
}

/**
* @name        : getPointCriteria
* @description : 기아 멤버스 포인트 사용 기준 정보 조회
* @date        : 2021.06.29
* @author      : 네이버시스템-주해원
*
* -Request Parameter
*   ci                  : CI (암호화)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*   useYn               : 기아 멤버스 포인트 사용여부
*   usePercent          : 기아 멤버스 포인트 사용비율
*/
var getPointCriteria = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.kiaPointCriteriaInfo.url,
        sendData : param,
        callback : function(receivedData){
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;
            var useYn = receivedData.pointUseYn;
            var usePercent = receivedData.pointUseRatio;

            if(code == "0000"){
                if(successFun != null){
                    successFun(useYn, usePercent);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
}

/**
* @name        : getPointCriteria
* @description : 기아 멤버스 포인트 사용 기준 정보 조회
* @date        : 2021.06.29
* @author      : 네이버시스템-주해원
*
* -Request Parameter
*   ci                  : CI (암호화)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*   useYn               : 기아 멤버스 포인트 사용여부
*   usePercent          : 기아 멤버스 포인트 사용비율
*/
var getPointCriteria = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.kiaPointCriteriaInfo.url,
        sendData : param,
        callback : function(receivedData){
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;
            var useYn = receivedData.pointUseYn;
            var usePercent = receivedData.pointUseRatio;

            if(code == "0000"){
                if(successFun != null){
                    successFun(useYn, usePercent);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
}

/**
* @name        : uptPointCriteria
* @description : 기아 멤버스 포인트 사용 기준 정보 변경
* @date        : 2021.06.29
* @author      : 네이버시스템-주해원
*
* -Request Parameter
*   ci                  : CI (암호화)
*   useYn               : 기아 멤버스 포인트 사용여부
*   usePercent          : 기아 멤버스 포인트 사용비율
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*/
var uptPointCriteria = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.uptKiaPointCriteria.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
}

/**
* @name        : reqEvChrgrResv
* @description : 충전기 예약 신청 요청
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   ci                   : CI (암호화)
*   sid                  : 충전소ID
*   cid                  : 충전기ID
*   reservationDate      : 예약날짜 (yyyy-MM-dd)
*   reservationTime      : 예약시간 (HH:mm)
* -Response
*   resultCode          : 결과코드
*   resultMessage       : 결과메세지
*   responseDate        : 응답시간
*   reservationNo       : 예약번호
*/
var reqEvChrgrResv = function(param, successFun, failFun){

    MNet_SSC.httpSend({
        path : SSC_API.reqEvChrgrResv.url,
        sendData : param,
        callback : function(receivedData){
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
* @name        : evChrgrResvReqList
* @description : 충전기 예약 신청 내역 조회
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   ci                  : CI (암호화)
*   currentPageNo       : 현재 페이지 번호
*   recordsPerPage      : 출력할 데이터 갯수
*   startDate           : 예약날짜 (yyyy-MM-dd)
*   endDate             : 예약시간 (HH:mm)
* -Response
*   resultCode              : 결과코드
*   resultMessage           : 결과메세지
*   responseDate            : 응답시간
*   responseListLength      : 응답리스트 길이
*   reservationInfoList     : 예약정보
*       reservation                : 예약번호
*       sid                        : 충전소 ID
*       cid                        : 충전기 ID
*       name                       : 충전소명
*       daddr                      : 주소
*       daddrDtl                   : 상세주소
*       connectType                : 충전기타입 (A-AC3상, D-DC차데모, C-DC콤보, 1-완속, 2-완속)
*       useStartTime               : 이용가능시작시간
*       useEndTime                 : 이용가능종료시간
*       reservationDate            : 예약날짜 yyyy-MM-dd
*       reservationTime            : 예약 시간 HH:mm
*       reservationStatusCode      : 예약 상태코드
*       reservationStatusCodeName  : 예약 상태 코드명
*       elctcAmount                : 충전 금액
*/
var evChrgrResvReqList = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.evChrgrResvReqList.url,
        sendData : param,
        callback : function(receivedData){
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
* @name        : canclEvChrgrResv
* @description : 충전기 예약 신청 취소
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   reservationNo           : 예약번호
* -Response
*   resultCode              : 결과코드
*   resultMessage           : 결과메세지
*   responseDate            : 응답시간
*
*/
var canclEvChrgrResv = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.canclEvChrgrResv.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
* @name        : evChrgrResvSttusList
* @description : 충전기 예약 현황 조회
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   sid                  : 충전소 ID
*   cid                  : 충전기 ID
*   reservationDate      : 예약날짜 yyyy-MM-dd
* -Response
*   resultCode              : 결과코드
*   resultMessage           : 결과메세지
*   responseDate            : 응답시간
*   possibleStartTime       : 충전기 운영 시작 시간 HH:mm
*   possibleEndTime         : 충전기 운영 종료 시간 HH:mm
*   reservationStatusInfo   : 예약 현황 정보
*       no                       : 순번
*       occupancyStartTime       : 예약 점유 시작 시간 HH:mm
*       occupancyEndTime         : 예약 점유 종료 시간 HH:mm
*/
var evChrgrResvSttusList = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.evChrgrResvSttusList.url,
        sendData : param,
        callback : function(receivedData){
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
* @name        : possibletime
* @description : 충전기 예약 가능 시간 조회
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   sid                  : 충전소 ID
*   cid                  : 충전기 ID
*   reservationDate      : 예약날짜 yyyy-MM-dd
* -Response
*   resultCode                      : 결과코드
*   resultMessage                   : 결과메세지
*   responseDate                    : 응답시간
*   possibleReservationTimeInfo     : 예약 가능 시간 리스트
*
*/
var possibletime = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.possibletime.url,
        sendData : param,
        callback : function(receivedData){
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};

/**
* @name        : reqResvStatus
* @description : 충전기 예약 신청 상태 조회
* @date        : 2021.09
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*   reservationNo        : 예약 번호
* -Response
*   resultCode                      : 결과코드
*   resultMessage                   : 결과메세지
*   responseDate                    : 응답시간
*   reservationStatusCode           : 예약 상태 코드
*
*/
var reqResvStatus = function(param, successFun, failFun){
    MNet_SSC.httpSend({
        path : SSC_API.reqResvStatus.url,
        sendData : param,
        callback : function(receivedData){
            var code = receivedData.resultCode;
            var msg = receivedData.resultMessage;

            if(code == "0000"){
                if(successFun != null){
                    successFun(receivedData);
                }
            }else {
                if(failFun != null){
                    failFun(code, msg);
                }
            }
        }
    });
};
// Straffic 통신
var MNet_SSC = {
    httpSend : function(options){
        if(options.indicatorYn != "N"){
            exWNShowIndicator();
        }
//        console.log("MNet_SSC accesstoken:" + PGV.loginInfo.accessToken);
        //console.log("MNet_SSC path :"+options.path);

        M.net.http.send({
            server : CONFIG.IS_DEV ? "HTTP_STRAFFIC_DEV" : "HTTP_STRAFFIC_REAL",
            path : options.path,
            method : CONSTANT.NET_MTHD.POST,
            timeout : options.timeout || (CONFIG.IS_DEV ? 150000 : 150000),
            data : options.sendData || {},
//            			userData : PGV.loginInfo ? {accessToken : PGV.loginInfo.accessToken, commonRest : true} : {commonRest : true},
            userData :  {isDev : CONFIG.IS_DEV} ,
            indicator : {
                show : false
            },
            success : function(receivedData, setting){
                //console.log("param: "+JSON.stringify(options.sendData));
                //console.log("response: "+JSON.stringify(receivedData));
                //exWNHideIndicator();
                var cleanXSSData = JSON.parse(cleanXSS(JSON.stringify(receivedData)));
                if (Object.keys(receivedData).length == 0) {
                    cleanXSSData = "";
                }

                if (options.callback) {
                    options.callback(cleanXSSData);
                }
            },

            error : function(errorCode, errorMessage, setting){
                console.log(errorCode+" "+errorMessage);
                exWNHideIndicator();
                sendSTrafficApiErrorLog(options.path, errorCode, errorMessage, options.backup, options.sendData, PGV.loginInfo ? PGV.loginInfo.accessToken : "");
                // 추후 공통처리가 필요한 경우 해당 부분에 처리
                if (options.errorCallback) {
                    options.errorCallback(errorCode, errorMessage);
                } else {
                    MPopup.alert(CONSTANT.NETWORK_ERR_MSG);
                }

                console.log("STraffic network error: code=" + errorCode + ", msg=" + errorMessage + ", path=" + options.path);
            }
        });
    }
};

/**
* @name        : callNav
* @description : 네비게이션 호출
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
*/
var callNav = function(app, name, lng, lat){

    if(lng == "" || lng == null || lng == undefined || lng == 'null'
    || lat == "" || lat == null || lat == undefined || lat == 'null'){
        MPopup.alert("해당 충전소의 위치 정보가 없습니다.");
        return;
    }

    if(app == 't'){
         // 앱버전 chk 및 앱버전 업데이트 유도
        var _os = PGV.device.os.name.toLowerCase();
        var _appVersion = Number(PGV.appInfo.app.version.replace(/[^0-9]/g,"").replace(/(\d{3})(\d*)/, "$1"));	// 앱 버전
        console.log(_os,_appVersion);
        tmapNav(name, lng, lat);
    }else if(app = 'k'){
        callKakaoNavi(name, lat, lng);
    }else{
        MPopup.alert("해당 네비게이션 앱정보가 없습니다.");
        return ;
    }


}

/**
* @name        : tmapNav
* @description : tmap 네비게이션 호출
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
*/
var tmapNav = function(name, lng, lat){
     console.log('apiKey',name, lng, lat);

   var _tmapApiKey = '';
    _tmapApiKey = CONFIG.IS_DEV ? CONSTANT.TMAP_API_DEV_KEY : CONSTANT.TMAP_API_REAL_KEY;

    if(CONFIG.TMAP_SDK_YN == "Y")
        exWNTMapNavigateSdk(_tmapApiKey, name, lng, lat);
    else
        exWNTMapNavigate(name, lng, lat);

    /** sdk ver */
    //exWNTMapNavigateSdk(_tmapApiKey, name, lng, lat);

}

/**
* @name        : onTMapErrorInfo
* @description : tmap api 에러 응답
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
*/
var onTMapErrorInfo = function(errCd, errMsg){
    console.log(errCd, errMsg);

    sendTMapErrorLog(errCd, errMsg);

    M.pop.alert({
        title : "알림 ("+errCd+")",
        message : '처리 중 문제가 발생했습니다. 다시 시도해주세요.',
        buttons: ["확인"],
//        buttons : ["취소", "확인"],
//        callback : function(idx){
//        }
    });
}

/**
* @name        : schEvChargerStation
* @description : EV 충전소 정보 조회
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*    bids (Array[string]): 충전소 ,
*    lat (string): 위도 ,
*    lng (string): 경도 ,
*    nowPage (integer): 현재 페이지 ,
*    pageSize (integer):  페이지 내 컨텐츠 수 ,
*    schGubun (string): 검색구분(01:이름,02:좌표) ,
*    schName (string): 검색어 ,
*    schRange (string): 검색범위,
*    types (Array[string]): 타입 (A:DC차 데모,B:DC콤보,C:승용차 AC완속,D:AC3상,E:AC급속3상)
* -Response
*    SchEvChargerStationResDto {
*       countNum (string): 충전기 목록 ,
*       currentPageNo (integer): 현재 페이지 번호 지정(1부터 시작) ,
*       pageEnd (integer, optional): 페이지 끝 번호 ,
*       pageSize (integer, optional): 한 페이지에 보여주는 레코드 수 ,
*       pageStart (integer, optional): 페이지 시작 번호 ,
*       resultCode (string): 처리 결과 코드 ,
*       resultMsg (string): 처리 결과 메시지 ,
*       schEvChargerStationListResDto (Array[SchEvChargerStationListResDto]): 충전소 목록 ,
*       totCnt (integer, optional): 전체 레코드 개수
*    }
*    SchEvChargerStationListResDto {
*       addr (string): 지번주소 ,
*       addrDtl (string): 지번주소 상세 ,
*       bCall (string): 운영기관 연락처 ,
*       bMngId (string): 충전기 기관내부관리ID ,
*       bName (string): 운영기관 명칭 ,
*       bid (string): 기관ID[ME 환경부 /SE 서울시 /KN 부산울산경남지역본부 /KP 한국전력 /HE 한국전기차충전서비스 /PI 포스코ICT /DE 대구환경공단 /EV 에버온 /GN 지엔텔 /KT KT /(이하 생략)] ,
*       chargerNum (string): 충전기 수 ,
*       dAddr (string): 도로명주소 (필수,없는 경우 지번) ,
*       dAddrDtl (string): 도로명주소 상세 ,
*       free (string): 주차비 무료 여부 ,
*       freeDtl (string): 주차비 상세 정보 ,
*       kind (string): 충전소구분 코드 [10 공공시설 /20 주차시설 /30 휴게시설 /40 관광시설 /50 상업시설 /60 차량정비시설 /70 기타시설] ,
*       kindDtl (string): 충전소구분 상세 코드 [1001 시청 /1002 구청 /1003 군청 /1004 주민센터(면사무소) /1005 보건소 /1006 지자체 시설 /1007 복지관 /1008 공공기관 /1009 도서관 /1010 의회 /1011 도청 /1012 체육관 /1099 공공기타 /(이하 생략)] ,
*       lat (string): 위도 ,
*       lng (string): 경도 ,
*       name (string): 이름 ,
*       resultCode (string): 처리 결과 코드 ,
*       resultMsg (string): 처리 결과 메시지 ,
*       sid (string): 충전소ID ,
*       status (string): 충전소 상태 ,
*       type (string): 충전기 타입 ,
*       useTime (string): 이용시간 ,
*       zcode (string): 지역코드 (법정동 시도 2자리) [11 서울특별시 /26 부산광역시 /27 대구광역시 /28 인천광역시 /29 광주광역시 /30 대전광역시 /31 울산광역시 /36 세종특별자치시 /41 경기도 /42 강원도 /43 충청북도 /44 충청남도 /45 전라북도 /46 전라남도 /47 경상북도 /48 경상남도 /50 제주특별자치도]
*    }
*/
var schEvChargerStation = function(param, successFun, failFun){
    MNet.httpSend({
        path : inf.evCharger.schEvChargerStation.url,
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
        	var code = receivedData.resultCode;
        	var msg = receivedData.resultMsg;

            if(code == "200"){
                successFun(receivedData);
            }else{
                failFun(code, msg);
            }
        }
    });
};

var schEvChargerStationIndN = function(param, successFun, failFun){
    MNet.httpSend({
        path : inf.evCharger.schEvChargerStation.url,
        indicatorYn : "N",
        sendData : param,
        callback : function(receivedData){
            exWNHideIndicator();
        	var code = receivedData.resultCode;
        	var msg = receivedData.resultMsg;

            if(code == "200"){
                successFun(receivedData);
            }else{
                failFun(code, msg);
            }
        }
    });
};

/**ㄿ
* @name        : schEvChargerStatus
* @description : EV 충전기 상태 정보 조회
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*    bid (string): 기관ID ,
*    sid (string): 충전소ID
*
* -Response
*    SchEvChargerStatusResDto {
*        countNum (string): 충전기 목록 ,
*        resultCode (string): 처리 결과 코드 ,
*        resultMsg (string): 처리 결과 메시지 ,
*        schEvChargerListResDto (Array[SchEvChargerStatusListResDto]): 충전기 목록
*    }
*    SchEvChargerStatusListResDto {
*        bid (string): 기관ID[ME 환경부 /SE 서울시 /KN 부산울산경남지역본부 /KP 한국전력 /HE 한국전기차충전서비스 /PI 포스코ICT /DE 대구환경공단 /EV 에버온 /GN 지엔텔 /KT KT /(이하 생략)] ,
*        cable (string): 충전 케이블 유무 ,
*        cid (string): 충전기ID ,
*        fee (string): 기본요금 (1kWh) ,
*        member (string): 멤버십여부 ,
*        note (string): 특이사항 ,
*        pay (string): 충전요금 유료 여부 ,
*        reserv (string): 예약 가능 여부 ,
*        resultCode (string): 처리 결과 코드 ,
*        resultMsg (string): 처리 결과 메시지 ,
*        show (string): 충전기 표시여부 ,
*        sid (string): 충전소ID ,
*        status (string): 충전기 상태 [0 알수없음 /1 통신이상 /2 충전대기 /3 충전중 /4 운영중지 /5 점검중 /6 예약중 /9 상태미확인 ] ,
*        type (string): 충전기타입 코드 [01 DC차데모 /02 승용차 AC완속 /03 DC차데모+AC3상 /04 DC콤보 /05 DC차데모+DC콤보 /06 DC차데모+AC3상+DC콤보 /07 AC급속3상] ,
*        updateTime (string): 충전기 업데이트 시간 ,
*        useTime (string): 이용가능시간
*    }
*/
var schEvChargerStatus = function(param, successFun, failFun){
     MNet.httpSend({
         path : inf.evCharger.schEvChargerStatus.url,
         sendData : param,
         callback : function(receivedData){
             exWNHideIndicator();
            var code = receivedData.resultCode;
            var msg = receivedData.resultMsg;

             if(code == "200"){
                 successFun(receivedData);
             }else{
                 failFun(code, msg);
             }
         }
     });
 };

/**
* @name        : schEvChargerFavoriteList
* @description : EV 충전소 즐겨찾기 리스트 조회
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*    currentPageNo (integer): 페이지번호( 처음페이지불러올때파라미터값없어도됨 ) ,
*    mbrId (string): 회원아이디 ,
*    pageSize (integer): 한 페이지에 보여주는 레코드 수
* -Response
*   SchEvChargerFavoriteResDto {
*       currentPageNo (integer): 현재 페이지 번호 지정(1부터 시작) ,
*       evChargerFavoriteList (Array[SchEvChargerFavoriteListResDto]): 즐겨찾기 목록 ,
*       pageEnd (integer, optional): 페이지 끝 번호 ,
*       pageSize (integer, optional): 한 페이지에 보여주는 레코드 수 ,
*       pageStart (integer, optional): 페이지 시작 번호 ,
*       resultCode (string): 처리 결과 코드 ,
*       resultMsg (string): 처리 결과 메시지 ,
*       totCnt (integer, optional): 전체 레코드 개수
*   }
*   SchEvChargerFavoriteListResDto {
*       addr (string): 지번주소 ,
*       addrDtl (string): 지번주소 상세 ,
*       bCall (string): 운영기관 연락처 ,
*       bMngId (string): 충전기 기관내부관리ID ,
*       bName (string): 운영기관 명칭 ,
*       bid (string): 기관ID[ME 환경부 /SE 서울시 /KN 부산울산경남지역본부 /KP 한국전력 /HE 한국전기차충전서비스 /PI 포스코ICT /DE 대구환경공단 /EV 에버온 /GN 지엔텔 /KT KT /(이하 생략)] ,
*       chargerNum (string): 충전기 수 ,
*       creDtm (string): 생성일 ,
*       dAddr (string): 도로명주소 (필수,없는 경우 지번) ,
*       dAddrDtl (string): 도로명주소 상세 ,
*       free (string): 주차비 무료 여부 ,
*       freeDtl (string): 주차비 상세 정보 ,
*       kind (string): 충전소구분 코드 [10 공공시설 /20 주차시설 /30 휴게시설 /40 관광시설 /50 상업시설 /60 차량정비시설 /70 기타시설] ,
*       kindDtl (string): 충전소구분 상세 코드 [1001 시청 /1002 구청 /1003 군청 /1004 주민센터(면사무소) /1005 보건소 /1006 지자체 시설 /1007 복지관 /1008 공공기관 /1009 도서관 /1010 의회 /1011 도청 /1012 체육관 /1099 공공기타 /(이하 생략)] ,
*       lat (string): 위도 ,
*       lng (string): 경도 ,
*       mbrId (string): 회원아이디 ,
*       name (string): 이름 ,
*       sid (string): 충전소ID ,
*       zcode (string): 지역코드 (법정동 시도 2자리) [11 서울특별시 /26 부산광역시 /27 대구광역시 /28 인천광역시 /29 광주광역시 /30 대전광역시 /31 울산광역시 /36 세종특별자치시 /41 경기도 /42 강원도 /43 충청북도 /44 충청남도 /45 전라북도 /46 전라남도 /47 경상북도 /48 경상남도 /50 제주특별자치도]
*    }
*/
var schEvChargerFavoriteList = function(param, successFun, failFun){
  MNet.httpSend({
      path : inf.evCharger.schEvChargerFavoriteList.url,
      sendData : param,
      callback : function(receivedData){
          exWNHideIndicator();
         var code = receivedData.resultCode;
         var msg = receivedData.resultMsg;

          if(code == "200"){
              successFun(receivedData);
          }else{
              failFun(code, msg);
          }
      }
  });
};

/**
* @name        : evChargerFavoriteReg
* @description : EV 충전소 즐겨찾기 [등록/삭제]
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* -Request Parameter
*    bid (string): 기관ID ,
*    mbrId (string): 회원아이디 ,
*    regGubun (string): 구분[등록:I/삭제:D] ,
*    sid (string): 충전소ID
* -Response
*   HeaderResDto {
*       resultCode (string): 처리 결과 코드 ,
*       resultMsg (string): 처리 결과 메시지
*   }
*/
var evChargerFavoriteReg = function(param, successFun, failFun){
  MNet.httpSend({
      path : inf.evCharger.evChargerFavoriteReg.url,
      sendData : param,
      callback : function(receivedData){
          exWNHideIndicator();
         var code = receivedData.resultCode;
         var msg = receivedData.resultMsg;

          if(code == "200"){
              successFun(receivedData);
          }else{
              failFun(code, msg);
          }
      }
  });
};

/**
* @name        : combEvChargerType
* @description : 모든 충전가능타입 조합
* @date        : 2021.08
* @author      : 네이버시스템-강보경
*
* '01': EV_CHARGER_TYPE[0]
* '02': EV_CHARGER_TYPE[1]
* '03': EV_CHARGER_TYPE[0],EV_CHARGER_TYPE[3]
* '04': EV_CHARGER_TYPE[2]
* '05': EV_CHARGER_TYPE[0],EV_CHARGER_TYPE[2]
* '06': EV_CHARGER_TYPE[0],EV_CHARGER_TYPE[2],EV_CHARGER_TYPE[3]
* '07': EV_CHARGER_TYPE[3]
*/
var combEvChargerType = function(param){


    var typeSwt =[false, false, false, false];

    if(param == null || param == undefined)
        return typeSwt;


    var type = param.split(",");

    for(var j = 0; j <type.length; j++){

        if(type[j] == '01' || type[j] == '03' || type[j] == '05' || type[j] == '06')
            typeSwt[0] = true;

        if(type[j] == '02')
            typeSwt[1] = true;

        if(type[j] == '04' || type[j] == '05' || type[j] == '06')
            typeSwt[2] = true;

        if(type[j] == '03' || type[j] == '06' || type[j] == '07')
            typeSwt[3] = true;
    }

    return typeSwt;
}

/**
  * @name        : detailEvSTMapPage
  * @description : 충전소 상세보기 페이지 이동
  * @date        : 2021.08
  * @author      : 네이버시스템-강보경
  *
  */
var detailEvSTMapPage = function(sid, name, lng, lat){
    console.log('detail',sid, name, lng, lat);

    if(lat == null || lat == '' || lat == undefined || lat == 'null'
    || lng == null || lng == '' || lng == undefined || lng == 'null' ){
        MPopup.alert("위치정보가 존재하지 않습니다.");
        return false;
    }
    // console.log(name, lat, lng);
    MPage.html({
        url : PATH_MANAGER.KIA_RNW_580_0000.path,
        animate : "MODAL_UP",
        param : {
            sid : sid,
            name : name,
            lat : lat,
            lng : lng
        }
    });
}

/**
  * @name        : schEvOprtList
  * @description : 충전소 운영기관 코드 리스트 조회
  * @date        : 2021.08
  * @author      : 네이버시스템-주해원
  *
  * -Request Parameter
  *    ccd (string): 옵션 전체 코드 조회 시 null 전달,
  *    ccdClsId (string): 전기차 충전소 운영기관 전체코드
  *
  */
var schEvOprtList = function(successFun, failFun){

  MNet.httpSend({
      path : inf.code.selectList.url,
      sendData : {
        ccd : null,
        ccdClsId : "10070",
      },
      callback : function(receivedData){
         exWNHideIndicator();
         var code = receivedData.resultCode;
         var msg = receivedData.resultMsg;

          if(code == "200"){
              successFun(receivedData);
          }else{
              failFun(code, msg);
          }
      }
  });
}