/**
 * @파일명 : ui_rnw.js
 * @작성자 : 박경윤
 * @생성일 : 2022. 02. 10.
 * @메뉴명 : 퍼블작업 공통 UI스크립트
 */

var RNW_UI = (function($) {
	'use strict';

	return {
		popup : {
			btnClose: '.dim, .js-popup-close-btn'
		},
		popupLayer: function(inOption) {
			/** inOption
			 * {
			 * 	selId : Layer Selector
			 * }
			 */

			var $layer = $(inOption.selId);
			var $layerCloseBtn = this.popup.btnClose;

			$layer.fadeIn('fast');
			$('body').addClass('js-overflow-hidden');

			$layer.find($layerCloseBtn).click(function(e) {
				e.preventDefault();
				var isBackKeyDisabled = $layer.attr("data-disable-back-key") === "Y";
				if(!isBackKeyDisabled){
					$layer.fadeOut('fast');
					$('body').removeClass('js-overflow-hidden');
					$(".wrapper").removeClass("donemove");
				}
			});

		},
		popupLayer_datepicker: function(inOption, obj) {
			/** inOption
			 * {
			 * 	selId : Layer Selector
			 * }
			 */
			var $obj;
			var $layer = $(inOption.selId);
			var $layerCloseBtn = this.popup.btnClose;
			var type = inOption.type;
			var inputDate;

			$layer.fadeIn('fast');

			// ------------------------------
			// 	Datepicker
			// ------------------------------

			//setDefaults
			$.datepicker.setDefaults({
				setDate: new Date(),
				dateFormat: 'yymmdd',
				prevText: '이전 달',
				nextText: '다음 달',
				monthNames: ['01월', '02월', '03월', '04월', '05월', '06월', '07월', '08월', '09월', '10월', '11월', '12월'],
				monthNamesShort: ['01월', '02월', '03월', '04월', '05월', '06월', '07월', '08월', '09월', '10월', '11월', '12월'],
				dayNames: ['일', '월', '화', '수', '목', '금', '토'],
				dayNamesShort: ['일', '월', '화', '수', '목', '금', '토'],
				dayNamesMin: ['일', '월', '화', '수', '목', '금', '토'],
				showMonthAfterYear: true,
				yearSuffix: '년',
				minDate: new Date("2018-11-01"), // 1년전까지 선택
				maxDate: 0, // 오늘날짜까지 선택
				onSelect: function(dateText, inst) {
					console.log('dateText', dateText)
					console.log('inst', inst)

					// 선택 날짜 화면 노출
					if($obj){
						var hTime = moment().format('HH');
						var tagTxt;

						//type에 따라 노출될 텍스트 상이
						if(type == 'A'){
							tagTxt = moment(dateText).format("YYYY.MM.DD");
						}else{
							tagTxt = moment(dateText).format("YYYY년 MM월 DD일");
						}

						$obj.find("input[type=text]").data('date', dateText + hTime);
						$obj.find("input[type=text]").val(tagTxt);
					}

					// 날짜가 선택되면 팝업이 닫힌다.
					$layer.fadeOut('fast');
					$('body').removeClass('js-overflow-hidden');
					$(".wrapper").removeClass("donemove");
					customPopup.remove(inOption.selId);
				}

			});

			//datepicker 생성
			$('.js-datepicker').datepicker();

			//obj가 존재하는 경우 해당 일자로 노출시키기 위함
			if(typeof obj != "undefined"){

				$obj = $(obj);
				//val 값에 대해 숫자만 추출
				var tmpStr = $(obj).find("input[type=text]").val();
				inputDate = tmpStr.replace(/[^0-9]/g, "");

				$('.js-datepicker').datepicker("setDate", inputDate);
			}

			// ------------------------------
			// ------------------------------

			$('body').addClass('js-overflow-hidden');

			$layer.find($layerCloseBtn).click(function(e) {
				e.preventDefault();

				$layer.fadeOut('fast');	
				$('body').removeClass('js-overflow-hidden');
				$(".wrapper").removeClass("donemove");// AS-IS
			});			

		},		
		popupLayer_selectBox: function(inOption) {
			/** inOption
			 * {
			 * 	selId : Layer Selector
			 * }
			 */

			var $layer = $(inOption.selId);
			var $layerCloseBtn = this.popup.btnClose;

			$layer.fadeIn('fast');
			$layer.find('.popup-wrap').addClass('is-ani');
			$('body').addClass('js-overflow-hidden');

			$layer.find($layerCloseBtn).click(function(e) {
				e.preventDefault();

				$layer.fadeOut('fast');
				$layer.find('.popup-wrap').removeClass('is-ani');
				
				$('body').removeClass('js-overflow-hidden');
			});
		},

	}

})((typeof jQuery === "object" && jQuery) || $);


// ------------------------------
// 	Setup
// ------------------------------
$(document).ready(function() {
	// ------------------------------

		fnSetup();
		fnIsFooterNav(); //footer 네비게이션 메뉴영역이 있는경우, 없는경우 제어
		
		// ===> MStatus.onReady(event);를 실행하기 위해, fnSetup()호출 위치를 mcore_api.js>onReadyCommon()로 옮김. (개발)

	// ------------------------------
});

$(window).on('load', function() {
	//console.log('load')
});

$(window).on('resize', function() {
	// location.reload();
});


// ------------------------------
function fnSetup() {

	// Common
	fnCheckDevice(); //디바이스 체크
	fnQuickTopBtn(); //최상단으로 가기 버튼
	fnIsFooterNav(); //footer 네비게이션 메뉴영역이 있는경우, 없는경우 제어
	
	// Element
	fnActiveBtn(); //버튼 활성화 버튼
	fnToggleBtn(); //토글 버튼
	fnTooltipBtn(); //툴팁 버튼

	fnCtrlInput(); //인풋 컨트롤
	fnInputClearBtn(); //인풋 clear버튼

	fnTabBtn() //탭
	fnAccodian(); //아코디언
}
// ------------------------------

// ------------------------------
// ------------------------------

function fnIsFooterNav(e) {

	if ($('.footer ul').length > 0) {
		$('.wrapper').addClass('is-footer-nav');
	}
	else {
		$('.wrapper').removeClass('is-footer-nav');
	}

}

function fnQuickTopBtn(e) {
	var $quickTopBtn = $('.js-quick-top-btn');

	$quickTopBtn.on('click', function(e) {
		e.preventDefault();
		$('html, body').stop().animate({scrollTop:0}, 300, 'swing');
	});
}

function fnCheckDevice() {
	var varUA = navigator.userAgent.toLowerCase(); //userAgent 값 얻기

	// 안드로이드
	if (varUA.indexOf('android') > -1) {
		return 'android';
	}
	// iOS
	else if (varUA.indexOf('iphone') > -1 || varUA.indexOf('ipad') > -1 || varUA.indexOf('ipod') > -1) {

		$('.footer').addClass('is-ios-border');

		return 'ios';
	}
	// 안드로이드, iOS 외
	else {
		return 'other';
	}
}

// ------------------------------
// ------------------------------


// ------------------------------
// 	Button
// ------------------------------
function fnActiveBtn(e) {
	var $activeBtn = $('.js-active-btn');

	$activeBtn.on('click', function(e) {
		e.preventDefault();

		$(this).addClass('active').siblings().removeClass('active');
	});
}

function fnToggleBtn(e) {
	var $toggleBtn = $('.js-toggle-btn');

	$toggleBtn.on('click', function(e) {
		e.preventDefault();
		$(this).closest('li').toggleClass('active');
	});
}

function fnTooltipBtn(e) {
	var $tooltipCtrl = $('.js-tooltip-ctrl');
	var $tooltipBtn = $('.js-tooltip-btn');
	var $tooltipBox = $('.js-tooltip-box');

	$tooltipBtn.on('click', function(e) {
		e.preventDefault();;
		$('body').addClass('js-tooltip-open');
		$tooltipCtrl.removeClass('active');
		$(this).closest($tooltipCtrl).toggleClass('active');
	});

	$tooltipBox.on('click', function(e) {
		e.preventDefault();
		$('body').removeClass('js-tooltip-open');
		$tooltipCtrl.removeClass('active');
	});
}


// ------------------------------
// 	Input
// ------------------------------
function fnCtrlInput(e) {
	var $inputCont = $('.input-cont');

	$inputCont.mouseenter(function(e) {

		// input이 읽기전용 일 경우
		if ($(this).find('input[type="text"]').prop('readonly') == true) {
			return false;
		}
		
		$(this).closest('.input-cont').addClass('js-input-focus--active');

	}).mouseleave(function() {
		$(this).closest('.input-cont').removeClass('js-input-focus--active');
	});

	$inputCont.find('input[type="text"]').on('keyup', function(e) {
		if (e.target.value) {
			$(this).closest('.input-cont').addClass('js-input-focus--active');
		}
		else {
			$(this).closest('.input-cont').removeClass('js-input-focus--active');
		}
	});
}

function fnInputClearBtn(e) {
	var $inputClearList = $('.js-input--clear');
	var $inputClearBtn = $('.js-input-clear-btn');

	$inputClearList.each(function(idx, item) {

		$(item).mouseenter(function(e) {

			// input이 읽기전용 일 경우
			if ($(this).find('input[type="text"]').prop('readonly') == true) {
				return false;
			}

			$(this).closest('.input-cont').addClass('js-input-focus--active');

			// input값이 비었으면 클리어버튼 비활성화
			if (!$(this).find('input[type="text"]').val() == '') {
				$(this).closest('.input-cont').addClass('js-input-clear--active');
			}
			else {
				$(this).closest('.input-cont').removeClass('js-input-clear--active');
			}

		}).mouseleave(function() {
			$(this).closest('.input-cont').removeClass('js-input-clear--active');
		});

		$(item).find('input[type="text"]').on('keyup', function(e) {
			if (e.target.value) {
				$(this).closest('.input-cont').addClass('js-input-clear--active');
			}
			else {
				$(this).closest('.input-cont').removeClass('js-input-clear--active');
			}
		});

		if (fnCheckDevice() == 'other') {
			$inputClearBtn.on('click', function(e) {
				$(this).closest('.input-cont').removeClass('js-input-clear--active');
				$(this).prev('input[type="text"]').val('');
			});
		}
		else {
			$inputClearBtn.on('touchstart', function(e) {
				$(this).closest('.input-cont').removeClass('js-input-clear--active');
				$(this).prev('input[type="text"]').val('');
			});
		}

	});



}

// ------------------------------
// 	Tab
// ------------------------------
function fnTabBtn(e) {
	var $tabBtn = $('.js-tab-btn');

	$tabBtn.on('click', function(e) {
		e.preventDefault();
		$(this).closest('li').addClass('active').siblings().removeClass('active'); // 20220311 케이스마텍 추헌남 - toggleClass를 addClass로 변경
	});
}

// ------------------------------
// 	Accodian
// ------------------------------
function fnAccodian(e) {
	var $accList = $('.js-accordion-list');
	var $accItem = $('.js-accordion-item');
	var $accBtn = $('.js-accordion-btn');
	var $accBody = $('.js-accordion-body');
	var $headerHeight = $('.header').outerHeight();
	var $filterHeight = $('.slider-filter-cont').outerHeight();

	$accList.find($accItem).each(function(idx, item) {
	
		$(this).find($accBtn).on('click', function(e) {
			e.preventDefault();

			// 아코디언 disabled
			if ($(this).hasClass('js-accordion-disabled')) {
				return false;
			}

			// 아코디언 작동
			$(this).closest($accItem).toggleClass('active');

			if ($(this).closest($accItem).hasClass('active')) {
				$(item).find($accBody).slideDown('fast');

				// 스크롤 위치 고정
				console.log($(this).closest($accItem).offset().top)
				if ($('.slider-filter-cont').is(':visible')) {
					$('html, body').stop().animate({scrollTop:$(this).closest($accItem).offset().top - ($headerHeight + $filterHeight)}, 300, 'swing');
				}
				else {
					$('html, body').stop().animate({scrollTop:$(this).closest($accItem).offset().top - ($headerHeight)}, 300, 'swing');
				}
									
			}
			else {
				$(item).find($accBody).slideUp('fast');
			}
		});
	});

}




