/**
 * @author 케이스마텍 추헌남
 * @description Bottom Tab Navigation 객체입니다.
 * @date 2022 02 25
 * @returns MyKiaBottomTab
 */


var MyKiaBottomTabInit = function () {
    /**
     * @name: Navigation Metadatas
     * @desc: Navigation 메타데이터를 보관합니다. 각 page 객체에서는 자동으로 currentDOM과 initialScript, htmlString 항목이 추가됩니다.
     */
    var navMeta = {
        //currentPage : "home", (현재 BottomTabNavigator가 보고 있는 페이지)
        notSignedIn: {
            home: {
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_110_0000,
                    tabularNavigators:[],
                    homeScript: function(){
                        loginHome.init();
                    },
                    currentDOM: null,
                    initialScript: null,
                }
            },
            carbuy: {
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_300_0000,
                    currentDOM: null,
                    initialScript: null,
                }
            },
            kiaStory:{
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_400_0000,
                    currentDOM: null,
                    initialScript: null,
                }
            },

            allMenu: {
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_600_0000,
                    tabularNavigators:[],
                    currentDOM: null,
                    initialScript: null,
                    interceptFunction: {
                        beforeRenderingHTML: function(htmlString){
                            var htmlObject = $('<div>' + htmlString + '</div>');
                            htmlObject = COMMON_FUNCTION.TOOL.UI.myKiaAllMenuInterface.setProfileSection(htmlObject);
                            htmlObject = COMMON_FUNCTION.TOOL.UI.myKiaAllMenuInterface.hideInaccessibleMenus(htmlObject);
                            htmlObject = COMMON_FUNCTION.TOOL.UI.myKiaAllMenuInterface.setBookmarkMenus(htmlObject);
                            return htmlObject.html();
                        }
                    }
                }
            },
        },
        signedIn: {
            home: {
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_120_0000,
                    tabularNavigators:[],
                    homeScript: function(){
                        /* 스크립트 리로드 방식
                        if (KIA_RNW_120_0000 && "function" == typeof KIA_RNW_120_0000.init) {
                            KIA_RNW_120_0000.init();
                        } else {
                            if (loginHome && "function" == typeof loginHome.init) {
                                loginHome.init();//todo 여기서 프리징 되버림.
                            }

                        }*/

                        /* MyKiaBottomTab . open 방식
                        if(isEmpty(MyKiaBottomTab)){
                            if (KIA_RNW_120_0000 && "function" == typeof KIA_RNW_120_0000.init) {
                                KIA_RNW_120_0000.init();
                            } else {
                                if (loginHome && "function" == typeof loginHome.init) {
                                    loginHome.init();//todo 여기서 프리징 되버림.
                                }

                            }
                        }else{
                            //todo 리로드 없이 그냥 open 하게했더니 html만 open하네?
                            MyKiaBottomTab.open("home",false);
                        }*/

                    },
                    currentDOM: null,
                    initialScript: null,
                }
            },
            mycar: {
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_210_0000,
                    tabularNavigators:[],
                    currentDOM: null,
                    initialScript: null,
                }
            },
            carbuy: {
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_300_0000,
                    currentDOM: null,
                    initialScript: null,
                }
            },
            EV:{
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_500_0000,
                    currentDOM: null,
                    initialScript: null,
                }
            },
            kiaStory:{
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_400_0000,
                    currentDOM: null,
                    initialScript: null,
                }
            },
            allMenu: {
                page: {
                    pathManager: PATH_MANAGER.KIA_RNW_600_0000,
                    tabularNavigators:[],
                    currentDOM: null,
                    initialScript: null,
                    interceptFunction: {
                        beforeRenderingHTML: function(htmlString){
                            var htmlObject = $('<div>' + htmlString + '</div>');
                            htmlObject = COMMON_FUNCTION.TOOL.UI.myKiaAllMenuInterface.setProfileSection(htmlObject);
                            htmlObject = COMMON_FUNCTION.TOOL.UI.myKiaAllMenuInterface.hideInaccessibleMenus(htmlObject);
                            htmlObject = COMMON_FUNCTION.TOOL.UI.myKiaAllMenuInterface.setBookmarkMenus(htmlObject);
                            return htmlObject.html();
                        }
                    }
                }
            },
        },

    }




    /**
     * @name: getCurrentNavMeta
     * @desc: 로그인 상황에 다른 NavMeta를 가져옵니다.
     */
    var getCurrentNavMeta = function () {
        // 상황에 따른 NavMeta 분기 후 return
        var loginInfo = null;
        try{
            loginInfo = PGV.loginInfo;
        }
        catch(e){
            loginInfo = MData.global('loginInfo');
        }

        var curNavMeta;
        if(!isEmpty(loginInfo)){
            curNavMeta = navMeta["signedIn"];
            var deleteMeta = "EV";
            try{
                if(!isEmpty(PGV.passData)){
                    var homeSelectCar = PGV.passData.car.selectCar;
                    if(!isEmpty(homeSelectCar)){
                        if(!isEmpty(homeSelectCar.memEvYn) && homeSelectCar.memEvYn!="N"){
                            deleteMeta="carbuy";
                        }
                    }
                }
                delete curNavMeta[deleteMeta];
            }catch(e){
                delete curNavMeta["EV"];
            }
        }else{
            curNavMeta = navMeta["notSignedIn"];
        }

        return curNavMeta;
        // return navMeta["notSignedIn"];
    }


    var setBottomTabButtonsEventListeners = function (currentBottomTopNavigation) {
        $(document).on("click", "[data-type]", function () {
            /* 케이스마텍 추헌남 - 하단 버튼에서 클릭했을 때에는 tabularNavigator가 항상 isMain:true인 부분을 로드하도록 수정 */
            /* (순서 중요, 해당 코드는 항상 이 이벤트 리스너의 최상위에 있어야 합니다.) */
            COMMON_FUNCTION.TOOL.NAVIGATOR_TABULAR.ignoreLoadingMain = false;
            
            var type = $(this).attr("data-type");
            
            /* html 페이지마다 기아스토리의 탭 이름이 다름... */
            if( type.includes("Story") ) type = "kiaStory";
            
            /* 같은 페이지에 있을 경우에는 다시 새로고침하지 않게 수정함 */
            var navCurrentPage = currentBottomTopNavigation.currentPage;
            if(navCurrentPage && navCurrentPage === type) return;


            currentBottomTopNavigation.open(type, true).catch(function (e) {
                COMMON_FUNCTION.TOOL.DEBUG.showAdvice('bottomTabNavigation : 탭 open에 실패하였습니다.');
                COMMON_FUNCTION.TOOL.DEBUG.showAdvice(e);
            });
        });
    }


    function uiFooter(tabName) {
        /* 보텀탭에서 ev와 차량 구매가 조건에 따라 토글된다. */

        var _$ev_btn = $("footer.footer ul li button.EV");
        var _$carbuy_btn = $("footer.footer ul li button.carbuy");
        if(!isEmpty(tabName)){
            _$ev_btn= $('[data-tab-name=mykia-tab-'+tabName+'] footer.footer ul li button.EV');
            _$carbuy_btn= $('[data-tab-name=mykia-tab-'+tabName+'] footer.footer ul li button.carbuy');
        }
        if (typeof PGV === "object"
            && PGV.passData
            && PGV.passData.car
            && PGV.passData.car["selectCar"]
            && PGV.passData.car["selectCar"]["memEvYn"]
            && PGV.passData.car.selectCar.memEvYn !== "N") {
            // if(!isEmpty(currentBottomTopNavigation.curNavMeta.EV)){
            _$carbuy_btn.closest("li").removeClass("none").addClass("none");
            if (isEmpty(_$ev_btn)) {
                _$carbuy_btn.closest("li").before('<li><button type="button" class="EV" data-type="EV">EV</button></li>');
            } else {
                _$ev_btn.closest("li").removeClass("none");
            }
        } else {
            _$carbuy_btn.removeClass("none");
            if (!isEmpty(_$ev_btn)) {
                _$ev_btn.closest("li").addClass("none");
            }
        }
    }

    /**
     * @name: Basic Navigation Structure
     * @desc: bottomTabNavigation 객체에 해당 값이 반환됩니다.
     */
    var navigationStructure = {
        // 상황에 따른 navMeta 분기 (함수화)
        curNavMeta: getCurrentNavMeta(),
        currentPage: "home", // 현재 BottomTabNavigator가 보고 있는 페이지
        lifeCycles: {
            beforeOpeningTheTab: null,
            afterInit: null,
            afterOpenedTheTab: null,
        },

        /**
         * @param {beforeOpeningTheTab, afterInit, afterOpenedTheTab: LifeCycle 함수} activateOption 
         */
        activate: function (activateOption) {
            COMMON_FUNCTION.TOOL.DEBUG.showTalk("bottomTabNavigation : 시작 중...");

            /* LifeCycle 함수 등록 */
            this.lifeCycles.beforeOpeningTheTab = activateOption && activateOption.beforeOpeningTheTab || null;
            this.lifeCycles.afterInit = activateOption && activateOption.afterInit || null;
            this.lifeCycles.afterOpenedTheTab = activateOption && activateOption.afterOpenedTheTab || null;

            /* 아래 작성된 함수 내부에서 해당 Navigation 객체 접근 위해 사용 */
            var currentBottomTopNavigation = this;
            uiFooter();

            /* 로그인 상황에 따른 NavMeta를 바탕으로 정보들을 저장합니다. */
            (function (curNavMeta) {
                return new Promise(function (resolve) {
                    // var tabViewArea = $('body');
                    var curNavMetaKeys = Object.keys(curNavMeta)
                    for (var i = 0; i < curNavMetaKeys.length; i++) {
                        var key = curNavMetaKeys[i];
                        var currentTab = curNavMeta[key];
                        var tabLoadingCounter = 0;
                        // home인 경우 
                        if (key === "home") {
                            currentTab.page["currentDOM"] = $('[data-tab-name=mykia-tab-home]');
                            currentTab.page["htmlString"] = $('[data-tab-name=mykia-tab-home]').prop('outerHTML');
                        }
                        else {
                            (function (key, currentTab) {
                                return new Promise(function (resolve) {
                                    var loadPage = $.get;
                                    loadPage(currentTab.page.pathManager.path, function (data) {
                                        var currentTabViewHTMLString = $(cleanXSS(data)).closest('.wrapper').prop('outerHTML');
                                        var currentTabViewDOM = $(currentTabViewHTMLString);
                                        currentTabViewDOM.addClass('is-section-line'); // !!! 케이스마텍 추헌남 - 문제 발생 시 삭제 필요
                                        currentTabViewDOM = currentTabViewDOM.attr('data-tab-name', 'mykia-tab-' + key);
                                        if( currentTab.page.interceptFunction &&
                                            typeof currentTab.page.interceptFunction.beforeRenderingHTML === "function" ){
                                            currentTab.page["htmlString"] = currentTab.page.interceptFunction.beforeRenderingHTML(currentTabViewDOM.prop('outerHTML'), key);
                                        }
                                        else{
                                            currentTab.page["htmlString"] = currentTabViewDOM.prop('outerHTML');
                                        }
                                        resolve({ "isTabWithoutDOMLoaded": true })
                                    })
                                })
                
                            })(key, currentTab)
                            .then(function () {
                                tabLoadingCounter++;
                            });
                        }
                    }

                    var timeoutCounter = 0;
                    var waitLoadingTabs = setInterval(function () {
                        timeoutCounter++;
                        if (timeoutCounter < 100){
                            if (curNavMetaKeys.length - 1 === tabLoadingCounter) {
                                clearInterval(waitLoadingTabs);
                                resolve({ "areTabsLoaded": true });
                            }
                        }
                        else{
                            clearInterval(waitLoadingTabs);
                            COMMON_FUNCTION.TOOL.DEBUG.showAdvice("bottomTabNavigation : 시작에 실패하였습니다... (탭 로딩 타임아웃)");
                            reject();
                        }
                    }, 100);
                })
            })(currentBottomTopNavigation.curNavMeta)
                .then(function () {
                setBottomTabButtonsEventListeners(currentBottomTopNavigation);
                // !!! 케이스마텍 추헌남 - open 함수 이후 렌더링이 다 되고 실행할 lifeCycle 함수
                var lcAfterInit = currentBottomTopNavigation.lifeCycles && currentBottomTopNavigation.lifeCycles.afterInit;
                lcAfterInit && typeof lcAfterInit === "function" && lcAfterInit();
                COMMON_FUNCTION.TOOL.DEBUG.showTalk("bottomTabNavigation : 시작 완료하였습니다...");
                exWNHideIndicator();
            });

        },

        open: function (tabName, isRefresh) {
            var currentBottomTopNavigation = this;
            return new Promise(function (resolve, reject) {
                try {
                    var isOpenSucceed = false;

                    var lcBeforeOpeningTheTab = currentBottomTopNavigation.lifeCycles && currentBottomTopNavigation.lifeCycles.beforeOpeningTheTab; 
                    var lcBeforeOpeningTheTabRun = typeof lcBeforeOpeningTheTab === "function" && 
                                                   lcBeforeOpeningTheTab ||
                                                   function(){return new Promise(function(resolve){resolve(true)})};
                    
                    lcBeforeOpeningTheTabRun(tabName)
                    .then(function(lcBeforeOpeningTheTabResult){
                        if(lcBeforeOpeningTheTabResult){
                            var currentTab = currentBottomTopNavigation.curNavMeta && currentBottomTopNavigation.curNavMeta[tabName] || null;
                            if (currentTab) {
                                if(currentTab.page && !currentTab.page.currentDOM || isRefresh){
                                    var tabViewArea = $('body');
                                    if (tabViewArea){
                                        if (isRefresh) {
                                            if(tabViewArea[0]){
                                                $('[data-tab-name=mykia-tab-' + tabName + ']').remove();
                                                if( currentTab.page.interceptFunction &&
                                                    typeof currentTab.page.interceptFunction.beforeRenderingHTML === "function"){
                                                    tabViewArea[0].innerHTML += currentTab.page.interceptFunction.beforeRenderingHTML(currentTab.page.htmlString, tabName);
                                                }
                                                else{
                                                    tabViewArea[0].innerHTML += currentTab.page.htmlString;
                                                }
                                                
                                            }
                                        } else{
                                            if(tabViewArea[0]) tabViewArea[0].innerHTML += currentTab.page.htmlString;
                                        }
                                    }
                                    currentTab.page["currentDOM"] = $('[data-tab-name=mykia-tab-' + tabName + ']');
                                    COMMON_FUNCTION.TOOL.DEBUG.showTalk("bottomTabNavigation : 해당 Tab 최초 open 혹은 refresh 하여 html 불러왔습니다.");
                                }
                                if (currentTab.page && currentTab.page.currentDOM && currentTab.page.currentDOM.length > 0) {
                                    var curNavMetaKeys = Object.keys(currentBottomTopNavigation.curNavMeta)
                                    for (var i = 0; i < curNavMetaKeys.length; i++) {
                                        var key = curNavMetaKeys[i];
                                        if (currentBottomTopNavigation.curNavMeta[key].page.currentDOM || isRefresh) {
                                            if (key !== tabName) {
                                                $('[data-tab-name=mykia-tab-' + key + ']').addClass('none');
                                            }
                                            else {
                                                if (key !== "home" && !currentBottomTopNavigation.curNavMeta[key].page.initialScript || isRefresh) {
                                                    if (tabName === "home"){
                                                        currentBottomTopNavigation.curNavMeta[key].page.homeScript && currentBottomTopNavigation.curNavMeta[key].page.homeScript();
                                                        COMMON_FUNCTION.TOOL.DEBUG.showTalk("bottomTabNavigation : 해당 Tab 최초 open 혹은 refresh 하여 initialScript 불러왔습니다.");
                                                    }
                                                    else{
                                                        var currentScriptFile = '../../js/ui_rnw/' + currentBottomTopNavigation.curNavMeta[key].page.pathManager.path.split("/")[1] + '/' + currentTab.page.pathManager.name + '.js';
                                                        var loadScript = $.getScript;
                                                        currentBottomTopNavigation.curNavMeta[key].page.initialScript = loadScript(currentScriptFile, function () {
                                                            COMMON_FUNCTION.TOOL.DEBUG.showTalk("bottomTabNavigation : 해당 Tab 최초 open 혹은 refresh 하여 initialScript 불러왔습니다.");
                                                        });
                                                    }
                                                }
                                                $('[data-tab-name=mykia-tab-' + tabName + ']').removeClass('none');
                                                isOpenSucceed = true;
                                            }
                                        }
                                    }
                                } else {
                                    COMMON_FUNCTION.TOOL.DEBUG.showAdvice("bottomTabNavigation : 해당 Tab 메타 정보는 있지만 DOM 객체를 찾을 수 없습니다 : " + tabName);
                                    reject();
                                }
                            }
                            else {
                                COMMON_FUNCTION.TOOL.DEBUG.showAdvice("bottomTabNavigation : 해당 Tab 메타 정보를 찾을 수 없습니다 : " + tabName);
                                reject();
                            }
                            // !!! 케이스마텍 추헌남 - 로그인 안했을 때 내차관리 들어가면 로그인 오류 띄우기
                            if(COMMON_FUNCTION.TOOL.AUTH.getCurrentPermission() === "non-login" && tabName === "mycar"){
                                COMMON_FUNCTION.TOOL.POPUP.toggle(PATH_MANAGER.KIA_RNW_080_0000.path, null, [null, function(){
                                    ccspLogin();
                                }]);
                                isOpenSucceed = false;
                                reject();
                            }
                            if (isOpenSucceed){
                                $(".footer li button").parent().removeClass("on active");
                                $('[data-type=' + tabName + ']').parent().addClass("on active");
                                uiFooter(tabName);
                                currentBottomTopNavigation.currentPage = tabName;
                            }
                            $('html, body').scrollTop(0);
                            if(tabName !== "mycar") fnSetup(); // !!! 케이스마텍 추헌남 - ui_rnw.js 의 fnSetup() 함수 실행 - mycar에서는 오류남
                            var lcAfterOpenedTheTab = currentBottomTopNavigation.lifeCycles && currentBottomTopNavigation.lifeCycles.afterOpenedTheTab;
                            lcAfterOpenedTheTab && typeof lcAfterOpenedTheTab === "function" && lcAfterOpenedTheTab(); // !!! 케이스마텍 추헌남 - open 함수 이후 렌더링이 다 되고 실행할 lifeCycle 함수
                            exWNHideIndicator();
                            resolve({ "isCurrentTabOpened": true });
                        }
                        else{
                            exWNHideIndicator();
                            resolve({ "isCurrentTabOpened": false });
                        }
                    })
                }
                catch (e) {
                    reject(e);
                }
            })
        },

        openListener: function (){
            var passData = M.data.global("passData");
            var bottomTabNavigator = passData && passData.bottomTabNavigator;
            var openListener =  bottomTabNavigator && bottomTabNavigator.openListener && bottomTabNavigator.openListener.pop();
            var tabName = openListener.tabName;
            var callback = openListener.callback;
            tabName && this.open(tabName) || COMMON_FUNCTION.TOOL.DEBUG.showAdvice('bottomTabNavigation : passData에서 tabName을 찾을 수 없습니다.');
            callback && callback();
            M.data.global("passData", passData);
        }
    }



    return $.extend({}, navigationStructure);
}
